#include <fstream>
#include <list>
#include "smesh.h"
#include "sfunction.h"
#include "complex.h"

using namespace std;

double minSubstract=0.1;
double intervalSubstract=1;
double precision = 1e-10;
double a0, b0; // popravi

dcomplex W(const dcomplex& z, const mesh1D& dom, const function1D<double>& De, const function1D<double>& dDe)
{
  double D0=0; int lb=dom.size(), ub=dom.size();
  if (z.imag()<minSubstract && z.real()>a0 && z.real()<b0) {
    D0 = De(dom.Interp(z.real()));
    lb = dom.find_(z.real()-intervalSubstract);
    ub = dom.find_(z.real()+intervalSubstract);
  }
  dcomplex sum = 0;
  for (int j=0; j<lb; j++) sum += dDe[j]/(z-dom[j]);
  for (int j=lb; j<ub; j++) sum += (dDe[j]-D0*dom.Dh(j))/(z-dom[j]);
  for (int j=ub; j<dom.size(); j++) sum += dDe[j]/(z-dom[j]);
  if (lb<dom.size() && ub<dom.size() && lb!=ub) sum += D0*log((z-dom[lb])/(z-dom[ub]));
  return sum;
}

bool ReadData(istream& input, mesh1D& om, function2D<dcomplex>& Sigma, int mdata)
{
  clog<<"Reading data with "<<om.size()<<" entries"<<endl;
  vector<double> data(mdata);
  int i=-1;
  while (input && ++i<om.size()){
    for (int j=0; j<mdata; j++) input>>data[j];
    input.ignore(500,'\n');
    om[i] = data[0];
    for (int j=0; j<mdata/2; j++) Sigma[j][i] = dcomplex(data[2*j+1], data[2*j+2]);
  }
  return true;
}

bool ReadDos(istream& input, mesh1D& dom, function1D<double>& De)
{ 
  clog<<"Reading DOS with "<<dom.size()<<" entries"<<endl;
  int i=-1;
  while (input && ++i<dom.size()){
    input>>dom[i];
    input>>De[i];
    input.ignore(500,'\n');
  }
  return true;
}

bool CheckStream(istream& inputf, int& n, int& m, bool read_mu, double& mu)
{
  istream input(inputf.rdbuf());
  input.seekg(0,ios::beg);
  
  string str; bool begincomm=false; n=0;
  getline(input,str); n++;
  if (!input.good()) {
    cerr << "ERROR: Wrong file format for hilbert or no data!" << endl;
    return false;
  }
  if (str.find('#')<string::npos) {
    begincomm=true;
    if (read_mu) ReadValue(mu,"mu",str);
    //    cout<<str<<endl;
    getline(input,str);
  };

  stringstream oneline;
  oneline << str << ends;
  m=0; double t;
  while (oneline){oneline>>t; m++;}
  m--;
  while (input){ getline(input,str); n++;}
  n--;
 
//    clog << " Number of entries: "<< n <<endl;
//    clog << " Number of columns: "<< m <<endl;
 
  inputf.seekg(0, ios::beg);
  if (begincomm) getline(inputf, str);
  if (!inputf){ cerr<<"Reopening didn't suceeded!"<<endl; return false;}
  return true;
}

int main (int argc, char *argv[], char *env[])
{
  int i=0; 
  double mu=0;
  list<string> files;
  while (++i<argc){
    string str(argv[i]);
    if (str=="-mu" && i<argc-1) mu = atof(argv[++i]);
    else {
      ifstream file(argv[i]);
      if (file) files.push_back(argv[i]);
    }
  }

  if (files.size()<2){
    cerr<<"Need input tb-DOS and Self-energy!... Exiting!"<<endl;
    clog<<"..............................................."<<endl;
    clog<<"*************** HILBERT TRANSFORM ******************\n";
    clog<<"**   hilbert transformation for complex function  **\n";
    clog<<"**       Copyright Kristjan Haule, 31.1.2003      **\n";
    clog<<"****************************************************\n";
    clog<<"\n";
    clog<<"Usage: "<<endl;
    clog<<"hilbert tbDOS Sigma.out \n" ;
    return 0;
  }

  ifstream dosstream, datastream;
  
  dosstream.open(files.begin()->c_str());
  datastream.open((++files.begin())->c_str());

  double dummy;
  int ndos, ndata, mdos, mdata;
  if (!CheckStream(dosstream,ndos,mdos,false,dummy)) exit(2);
  if (!CheckStream(datastream,ndata,mdata,true,mu)) exit(2);
  
  if (mdos<2){ cerr<<"Not enough columns in dos - file"<<endl;}

  
  clog<<"*************** PARAMETERS ************"<<endl;
  clog<<"Number of self-energy columns:      "<<mdata/2<<endl;
  clog<<"mu="<<mu<<endl;
  clog<<"****************************************"<<endl;
  clog<<"Output: om Delta1, Delta2,..."<<endl;
  clog<<"****************************************"<<endl;
  
  mesh1D dom(ndos);
  mesh1D om(ndata);
  function1D<double> De(ndos), dDe(ndos);
  
  if (!ReadDos(dosstream, dom, De)) {cerr<<"Wrong dos file"<<endl; exit(1);}
  dom.SetUp(0.0);
  double isum=0;
  for (int i=0; i<dom.size(); i++){
    dDe[i] = De[i]*dom.Dh(i);
    isum += dDe[i];
  }
  clog<<"integral is "<<isum<<endl;
  // renormalizing DOS to unity
  for (int i=0; i<dom.size(); i++){
    De[i] *= 1/isum;
    dDe[i] *= 1/isum;
  }
  // average <H>
  double aver_eps=0;
  for (int i=0; i<dom.size(); i++){
    aver_eps += dDe[i]*dom[i];
  }
  clog<<"average <eps>="<<aver_eps<<endl;
  
  int j=0; while (j<dom.size() && fabs(De[j])<1e-6) j++; a0 = dom[j<=0 ? 0 : j-1];
  j=dom.size()-1; while (j>0 && fabs(De[j])<1e-6) j--; b0 = dom[j>=dom.size()-1 ? dom.size()-1 : j+1];

  clog<<"a0="<<a0<<" b0="<<b0<<endl;

  function2D<dcomplex> Sigma(mdata/2, ndata);
  if (!ReadData(datastream, om, Sigma, mdata)) {cerr<<"Wrond data file"<<endl; exit(1);}

//   print(cout, om, Sigma[0], Sigma[1], 25);
//   return 0;

  om.SetUp(0.0);
  dcomplex G00, Delta, Sig;
  for (int i=0; i<om.size(); i++){
    cout<<setw(25)<<om[i]<<" ";
    dcomplex z(mu,om[i]);
    for (int j=0; j<Sigma.size_N(); j++){
      dcomplex xi = z-Sigma[j][i]+aver_eps;
      G00 = W(xi, dom, De, dDe);
      Delta = xi-1/G00-aver_eps;
      cout<<setw(25)<<Delta<<" ";
    }
    cout<<endl;
  }
  return 0;
}
